      program bfactors
c
c     Calcul des Facteurs B.
c    (pour les C-alpha)
c     Comparaison aux Facteurs B experimentaux.
c
c     A partir d'un sous-ensemble de vecteurs au format CERFACS.
c     Le fichier pdb donne les informations atomiques. 
c
c     AJOUTS A PREVOIR: 
c     Test du caractere croissant des frequences (si sous-ens)
c     Test de l'absence de "grand gap" dans les non-nulles.
c     All-atom correlation.
c     Calcul des anisotropies.
c     Distribution quantique.
c
c     YHS-Jan-2001: Premiere version (From rms_mode.f, v1.14).
c     YHS-Sep-2002: Derniere version.
 
      implicit none
      integer natmax, nvecmx
      parameter(natmax=15000,nvecmx=106)
      integer fatres(natmax), i, ii, imax, iresat(natmax), 
     .       j, jj, jmax, k, lnomeig, lnompdb, natom, ncalpha, nddl, 
     .       nexp, nok, nres, nskip, numvec(natmax), nunit, nvec, 
     .       nvecsel, nzero, uneig, unout, unpdb
      double precision bfact(natmax), bigzero, binfo(natmax), corr, 
     .       eigval(3*natmax), eigvalmin, kT, loweigval, 
     .       matvec(3*natmax,nvecmx), norme, moyp, moyx, rmsp, rmsx, 
     .       rsmall, twopi, wzero, xat(natmax), yat(natmax), zat(natmax)
      logical qerror, qexist, qinter, qok, qrt, qselvec(nvecmx)
      character atonam(natmax)*4, cformat*10, cstatus*10, namfil*64, 
     .       nomeig*64, nompdb*64, program*10, progrer*13, 
     .       progrwn*13, resnam(natmax)*4, version*40
 
      version=' Version 1.22, Bordeaux.'

c     Take rotations-translations into account ? 
      qrt=.false.
 
c     Unites Charmm (Angs, kcal/mole, m.a) et T=300K.
      kT=0.59
      twopi=2.d0*acos(-1.d0)
      rsmall=1e-04
 
      program=' Bfactors>'
      progrwn='%Bfactors-Wn>'
      progrer='%Bfactors-Er>'
 
      write(6,'(2A)') program,version
 
c     Instructions:
c     ------------
      call getnam('Eigenvector filename ?',nomeig,lnomeig,qok)
      if (.not.qok) stop
 
      nunit=10
      uneig=nunit
      nunit=nunit+1
      cformat="FORMATTED"
      cstatus="old"
      call openam(nomeig,cformat,cstatus,uneig,.true.,
     .            qinter,qexist)
      if (qinter.or..not.qexist) stop
 
      call getnam('Corresponding pdb filename ?',nompdb,lnompdb,qok)
      if (.not.qok) stop
 
      unpdb=nunit
      nunit=nunit+1
      cformat="FORMATTED"
      cstatus="old"
      call openam(nompdb,cformat,cstatus,unpdb,.true.,
     .            qinter,qexist)
      if (qinter.or..not.qexist) stop
 
      nskip=0
      call getnum('Number of skipped eigenvectors ?',nskip,
     .     -1,nvecmx,.true.,qok)
 
      nvecsel=56
      call getnum('Number of usefull eigenvectors ?',nvecsel,
     .     1,nvecmx,.true.,qok)
 
      wzero=-1
      if (qrt) then
      call getchi('Relative weight for Rotation-Translation modes ?',
     .     wzero,qok)
      endif
 
c     Lecture:
c     -------
      call rdatompdb(unpdb,' ',xat,yat,zat,binfo,
     .     atonam,iresat,resnam,natmax,natom,
     .     fatres,natmax,nres,qerror)
 
      call rdmodfacs(uneig,3*natmax,nvecmx,numvec,eigval,
     .               matvec,nddl,nvec)
 
      write(6,'(/A,I5,A,I6,A)') program,
     .  nvec,' vectors, ',nddl,' coordinates in file.'
 
      if (nddl.ne.3*natom) then
          write(6,'(/2A)') progrer,' Inconsistent vector and pdb files.'
          stop
      endif
  
      if (nvec.lt.nvecsel) then
          write(6,'(2A)') progrwn,
     .  ' All vectors required were not found in vector file.'
          nvecsel=nvec
      endif
 
c     Sortie:
c     ------
      namfil='bfactors.pred'
      unout=nunit
      nunit=nunit+1
      cformat="FORMATTED"
      cstatus="ove"
      call openam(namfil,cformat,cstatus,unout,.true.,
     .            qinter,qexist)
 
c     Compter les modes a Valeur propre "nulle":
c     a) Valeur propre la plus proche de la nullite.
c     b) Valeur propres "proches" de cette valeur (a un facteur 100 pres).
 
      eigvalmin=9.e+99
      do i=1,nvec
         if (i.gt.nskip) then
             qselvec(i)=.true.
         else
             qselvec(i)=.false.
         endif
         if (dabs(eigval(i)).lt.eigvalmin) eigvalmin=dabs(eigval(i))
      enddo 
      write(6,'(/2A,F13.6)') program,
     .' Best zero-eigenvalue found  :',eigvalmin
 
      nzero=0
      nok=0
      bigzero=eigvalmin
      loweigval=9.e+99
 
      do i=1,nvec
         if (dabs(eigval(i)).lt.100*eigvalmin) then
             if (dabs(eigval(i)).gt.bigzero) bigzero=dabs(eigval(i))
             nzero=nzero+1
             qselvec(i)=.false.
         else if (qselvec(i)) then
             nok=nok+1
             if (nok.gt.nvecsel) then
                 qselvec(i)=.false.
             else
                 if (dabs(eigval(i)).lt.loweigval) 
     .               loweigval=dabs(eigval(i))
             endif
         endif
      enddo 
 
      write(6,'(A,I6,A,F12.6)') program,nzero,
     . ' eigenvalues less than : ',bigzero
 
      if (nzero.lt.nvec) 
     .  write(6,'(2A,F13.6)') program,
     .' Lowest non-zero eigenvalue  :',loweigval
 
      if (wzero.le.0) then
          nok=min(nok,nvecsel)
          write(6,'(A,I6,A)') program,nok,
     .  ' eigenvectors will be considered.'
      else
          nok=min(nok,nvecsel)
          write(6,'(A,I6,A)') program,nok+nzero,
     .  ' eigenvectors will be considered.'
          if (nzero.eq.nvec) then 
              loweigval=bigzero+100.d0
              write(6,'(2A,F13.6)') progrwn,
     .      ' Eigenvalue assumed for each mode:',loweigval
          endif
      endif
      if (nskip.gt.0) then
          write(6,'(A,I6,A)') program,nskip,
     .  ' eigenvectors were skipped.'
      endif
 
c     Prediction (classique) et
c     Comparaison avec les donnees experimentales: 
c     -------------------------------------------
c     B-theorique = 8*PI**2*<ri**2>/3
c       <ri**2>   = kT*sum_j(aij**2/omegaj**2)
c     ou:
c     omegaj**2, est la j-eme valeur propre de la matrice et
c     ri, la i-eme coordonnee masse-ponderee.
 
      nexp=0
      ncalpha=0
      do i=1,natom
         if (binfo(i).lt.0.d0) nexp=nexp+1
         if (atonam(i)(1:3).eq.'CA ') ncalpha=ncalpha+1
      enddo
 
      if (nok.gt.0) then
      if (wzero.gt.0) then
        write(6,'(2A,F8.3)') program,
     .' Effective eigenvalue for Rotation-Tranlation modes:',
     .  loweigval/wzero
        do i=1,nvec
           if (.not.qselvec(i)) eigval(i)=loweigval/wzero
        enddo
      else
        write(6,'(2A)') program,
     .' Rotation-Tranlation modes are skipped.'
      endif
      endif
 
      if (nok.gt.0)
     .  write(6,'(/2A/10X,A)') program,
     .' Mass-weighted B-factors are computed.',
     .'(CHARMM units assumed for eigenvalues)'
 
      corr=0.d0
      moyp=0.d0
      moyx=0.d0
      rmsp=0.d0
      rmsx=0.d0
      do i=1,natom
         ii=3*i-2
         bfact(i)=0.d0
         if (nok.gt.0) then
         do j=1,nvec
         if (wzero.gt.0.or.qselvec(j)) then
         norme=matvec(ii,j)**2.d0+
     .         matvec(ii+1,j)**2.d0+
     .         matvec(ii+2,j)**2.d0
         bfact(i)=bfact(i)+norme/eigval(j)
         endif
         enddo
         bfact(i)=2*twopi**2*kT*bfact(i)/3
         endif
 
         if (nexp.eq.0.and.atonam(i)(1:3).eq.'CA ') then
             corr=corr+bfact(i)*binfo(i)
             moyp=moyp+bfact(i)
             moyx=moyx+binfo(i)
             rmsp=rmsp+bfact(i)*bfact(i)
             rmsx=rmsx+binfo(i)*binfo(i) 
         endif
      enddo
 
      if (ncalpha.gt.0) then
      moyp=moyp/dfloat(ncalpha)
      moyx=moyx/dfloat(ncalpha)
      rmsp=rmsp/dfloat(ncalpha)-moyp*moyp
      rmsx=rmsx/dfloat(ncalpha)-moyx*moyx
      rmsp=dsqrt(rmsp) 
      rmsx=dsqrt(rmsx) 
 
      if (nexp.eq.0) then
      if (rmsx.gt.rsmall) then
      corr=corr/dfloat(ncalpha)-moyp*moyx
      if (rmsp.gt.rsmall) corr=corr/(rmsp*rmsx)
 
      write(6,'(/2A,F10.3,A,I6,A)') program,
     .    ' Correlation= ',corr,' for ',ncalpha,' C-alpha atoms.'
      else
      write(6,'(/2A/)') progrwn,
     .    ' Experimental B-factors are nearly constant !'
      endif
      endif
 
      write(6,'(2A,F10.3,A,F6.2)') program,
     .    ' <B-predict>= ',moyp,' +/- ',rmsp
      write(6,'(2A,F10.3,A,F6.2)') program,
     .    ' <B-crystal>= ',moyx,' +/- ',rmsx
      write(6,'(2A,F10.3)') program,
     .    ' Shiftng-fct= ',moyx-moyp
      if (rmsp.gt.rsmall)
     .write(6,'(2A,F10.3)') program,
     .    ' Scaling-fct= ',rmsx/rmsp
 
      write(6,'(/2A)') program,
     .  ' Predicted, Scaled and Experimental B-factors '//
     .   'are saved.'
 
      else
      write(6,'(/A,I6,A)') progrwn,nexp,
     .' negative values found in the B-factors column of pdb file.'
      write(6,'(2A/10X,A)') progrwn,
     .' So, only mass-weighted predicted B-factors will be saved.',
     .'(CHARMM units assumed for eigenvalues)'
      endif
 
      do i=1,natom
         if (nexp.eq.0.and.rmsp.gt.rsmall) then
         write(unout,'(I6,1X,A4,1X,A4,1X,I5,3F12.4)') 
     .         i,atonam(i),resnam(i),iresat(i),
     .         bfact(i),(bfact(i)-moyp)*rmsx/rmsp+moyx,binfo(i)
         else
         write(unout,'(I6,1X,A4,1X,A4,1X,I5,F12.4)') 
     .         i,atonam(i),resnam(i),iresat(i),bfact(i)
         endif
      enddo
 
      write(6,'(/2A)') program,' Normal end.'
 
      stop
      end
 
      subroutine rdmodfacs(uneig,nddlmax,nvecmx,numvec,eigval,
     .           matvec,nddl,nvec)
c
c     Lecture de modes CERFACS.
c
c     Premieres versions (rdcerfacs):
c     YHS-Nov-1996.
c     Dernieres modifications:
c     YHS-Jan-2001.
cI/O:
      integer numvec(*), nvecmx, nddlmax, nvec, nddl, uneig
      double precision eigval(*), matvec(nddlmax,*)
cLocal:
      integer nmotsmax
      parameter(nmotsmax=100)
      integer nerr, ivec, indnm_cfacs, nmots,
     .        i, ii, j, jj, k, kk
      logical qfound, qold, qfirst
      character*1 carnum
      character*132 lign132, mots(nmotsmax)
cDefaut:
      nerr=0
      qfirst=.true.
      qold=.false.
      qfound=.false.
 100  continue
      read (uneig,'(A)',end=300,err=110) lign132
      goto 120
 110  continue
      nerr=nerr+1
 120  continue
c
      qfound=qfound.or.
     .      (index(lign132,' value ').gt.0.and.
     .       index(lign132,' vector ').gt.0.and.
     .       index(lign132,' residual ').le.0)
      qold=qold.or.
     .      (index(lign132,' VALUE ').gt.0.and.
     .       index(lign132,' VECTOR ').gt.0)
c
      if (.not.qfound.and..not.qold) goto 100
c________________________________________
c
c     Lecture des Valeur propres des modes :
c________________________________________
c
      if (qfirst) then
          if (qold) then
          write(6,'(/A)') 
     .  ' Rdmodfacs> Old Blzpack file format detected.'
          else
          write(6,'(/A)') 
     .  ' Rdmodfacs> Blzpack file format detected.'
          endif
          qfirst=.false.
      endif
c
      ivec=0
      nvec=0
 250  continue
      ivec=ivec+1
      if (ivec.gt.nvecmx) then
          write(6,'(/A,I5,A)') 
     .  '%Rdmodfacs-Wn> More than ',nvecmx,' vectors in file.'
          return
      endif
c
      read(lign132,'(7X,I5,12X,G12.4)',end=240,err=240)
     .     numvec(ivec), eigval(ivec)
c
      goto 255
 240  continue
      write(6,'(/3A)')
     .    '%Rdmodfacs-W> Pb with ligne: ',lign132(1:36),'...'
 255  continue
c
      nvec=ivec
      write(6,'(/A,I6)')
     .    ' Rdmodfacs> Numero du vecteur CERFACS en lecture:',
     .      numvec(ivec)
      write(6,'(A,1PG12.4)')
     .    ' Rdmodfacs> Valeur propre du vecteur en lecture:',
     .      eigval(ivec)
c
      if (numvec(ivec).le.0)
     .    write(6,'(/A/A)')
     .  '%Rdmodfacs-W> Vector number was expected in:',
     .    lign132
c
      read(uneig,'(A)',end=230,err=230) lign132
 230  continue
      read(lign132,'(1X,A1)',end=232,err=232) carnum
 232  continue
      if ((qfound.and.carnum.ne.'=').or.
     .    (qold.and.carnum.ne.'-')) then
          write(6,'(2A/A)')
     .       ' %Rdmodfacs-Warning> Unexpected character ',
     .       ' in second column of line:',
     .    lign132
      endif
c____________________________________________________
c
c     2) Lecture des coordonnees des modes CERFACS :
c        Format libre.
c____________________________________________________
c
      k=0
 257  continue
      if (k.ge.nddlmax) then
          write(6,'(/A,I6,A,I5)') 
     .  '%Rdmodfacs-Err> More than ',nddlmax,
     .  ' coordinates for vector ',ivec
          return
      endif
c
      read(uneig,'(A)',end=300,err=270) lign132
c
c     Nombre de coordonnees par ligne:
      call string_split(lign132,132,' ',
     .                  mots,nmotsmax,nmots)
c
      if (lign132.eq.' ') then
          read(uneig,'(A)',end=300,err=260) lign132
      else if (.not.qold.or.index(lign132,' VALUE ').le.0) then
          read(lign132,*,end=258)
     .   (matvec(k+ii,ivec),ii=1,nmots)
          k=k+nmots
          goto 257
 258      continue
      endif
      nddl=k
c
 260  continue
      indnm_cfacs=index(lign132,'       VALUE')
      if (indnm_cfacs.le.0)
     .    indnm_cfacs=index(lign132,'       value')
      if (indnm_cfacs.gt.0) then
          goto 250
      else
          write(6,'(A,A/A/A)')
     .  ' Rdmodfacs: Lecture des modes CERFACS terminee.',
     .  ' Item VALUE non trouve dans la ligne:',lign132
          goto 300
      endif
c
 270  continue
      write(6,'(A,I6,A)')
     .   ' %Rdmodfacs-Error: durant la lecture de la coordonnee ',
     .      i,' du mode.'
      stop
c
 220  continue
c*****Ligne suivante de la lecture du fichier des modes en cours :
c
      goto 100
c
c     Fin de la lecture du fichier des modes :
c
 300  continue
      return
      end
c
c     Lecture de noms de fichier + Ouverture.
c     Serie de routines utilitaires:
c     openam
c     stringcl
c     string_split
c     Version 1.11, Bordeaux.
c
      SUBROUTINE openam(namfil,cformat,cstatus,unit,qverbos,
     .                  qinterr,qexist)
c
c     Ouverture d'un fichier de nom NAMFIL, sur l'unite UNIT.
c
c     input:
c        namfil: nom du fichier a ouvrir. 
c        "stop", "end", "fin", "quit" : arretent le programme.
c        cstatus: mots-cles fortran... ou "OVE" pour overwrite.
c     output: 
c        qexist: flag / existence du fichier 
c        qinterr: Pas de nom pour le fichier cherche.
c
c     YHS-oct-1993: Premiere version.
c     YHS-jan-2000: Derniere modification.
c I/O:
      logical qinterr, qverbos, qexist
      integer unit
      character*10 cformat, cstatus
      character*64 namfil
c Local
      character*132 ordrunix
c begin:
      if (cstatus.eq.'old') cstatus='OLD'
      if (cstatus.eq.'new') cstatus='NEW'
      if (cstatus.eq.'ove') cstatus='OVE'
      if (cstatus.eq.'unknown') cstatus='UNKNOWN'
c
      qinterr=.false.
      qexist=.false.
c
      if (namfil.eq.' ') then 
          qinterr=.true.
          write(6,'(A)') '%Openam-Err> No filename.'
          return
      endif
c
      if (namfil.eq.'stop'.or.namfil.eq.'end'                         
     &    .or.namfil.eq.'fin'.or.namfil.eq.'quit') then 
         write(*,*) 'Openam> Program is stopping on user request.'
         stop                                                                   
      endif 
c
c     Checks if filename is consistent with the opening:
c
      inquire(file=namfil,exist=qexist)
      if (.not.qexist.and.cstatus.eq.'OLD') then
          qinterr=.true.
          write(6,'(A)') '%Openam-Err> File not found.'
          return
      endif
c
      if (qexist.and.cstatus.eq.'NEW') then
         write(*,'(/A)') 
     .      '%Openam-Err> This file exists:',namfil
         stop
      else if (qexist.and.cstatus.eq.'OVE') then
         ordrunix='rm '//namfil
         call system(ordrunix)
      endif
      if (cstatus.eq.'OVE') cstatus='NEW'
c                                                                   
      if (qverbos) then
         write(*,'(/A,I6,A)')
     .           ' Openam> file on opening on unit ',unit,':'
         write(*,*) namfil
      endif
      open(file=namfil,form=cformat,
     .     status=cstatus,unit=unit)                
c        
      return                                                                       
      end
c
      subroutine string_split(chaine,taille,delimiteur,
     .                        souschaine,nbremax,nbre)
c
c     "Chaine" est coupee en "nbre" "souschaine" de part et d'autre du
c     "delimiteur"
c      YHS-Sep-93, Uppsala
c I/O:
      integer taille, nbremax, nbre
      character*(*) chaine, souschaine(*), delimiteur
c Local:
      integer icar, iprev
c
      nbre=1
      iprev=1
      souschaine(1)=chaine
      do icar=1,taille
         if (chaine(icar:icar).eq.delimiteur) then
            if (icar-1.ge.iprev) then
               souschaine(nbre)=chaine(iprev:icar-1)
               nbre=nbre+1
               if (nbre.le.nbremax) then
                  if (icar+1.le.taille.and.
     .               chaine(icar+1:taille).ne.' ') then
                     souschaine(nbre)=chaine(icar+1:taille) 
                  else
                     nbre=nbre-1
                     return
                  endif
               else
                  write(6,'(A,I6,A/A)') 
     .               ' %String_split-Err: more than ',nbremax,
     .               ' substrings in : ',chaine
                  return
               endif
            endif
            iprev=icar+1
         endif
      enddo
c
      return
      end
c
      subroutine stringcl(chaine,nonblancs)
c
c     Les caracteres "blancs" de la CHAINE sont retires (a gauche et au milieu).
c     L'entier NONBLANCS donne la position du dernier caractere.
c
c     YHS-Jan-95, Toulouse.
c     YHS-Oct-00, Bordeaux.
c I/O:
      integer nonblancs
      character*(*) chaine
c Local:
      integer icar, ncar, taille
c Begin:
      nonblancs=0
      taille=len(chaine)
      if (taille.le.0) return
c
      if (index(chaine(1:taille),' ').le.0) then
          nonblancs=taille
          return
      endif
c
c*****Nettoyage des blancs a gauche.
c     Premier non-blanc:
c
      do icar=1,taille
         if (chaine(icar:icar).ne.' ') goto 150
      enddo
      icar=taille
 150  continue
      chaine=chaine(icar:taille)
c
c*****Nettoyage des blancs au milieu.
c
          icar=1
          ncar=1
 170      continue
          icar=icar+1
          ncar=ncar+1
          if (chaine(icar:icar).eq.' ') then
              chaine=chaine(1:icar-1)//chaine(icar+1:taille) 
              icar=icar-1
          endif
          if (ncar.lt.taille-1) goto 170
c
      nonblancs=index(chaine,' ')-1
c
      return
      end
c
      subroutine rdatompdb(unpdb,ssusel,xat,yat,zat,binfo,
     .           atonam,iresat,resnam,natmax,natom,
     .           fatres,nresmx,nres,qerror)
c
c     Lecture ligne a ligne d'un fichier pdb.
c     Uniquement les lignes commencant par 'ATOM'.
c     Uniquement ceux de la sous-unite selectionnee.
c
c     fatres(i): numero du premier atome du residu i.
c
c     YHS-nov-1996: premiere version (Toulouse).
c     YHS-jan-2001: derniere version (Bordeaux).
c
      implicit none
cI/O:
      integer unpdb, natmax, iresat(*), natom,  lnom,
     .        nresmx, nres, fatres(*)
      double precision xat(*), yat(*), zat(*), binfo(*)
      logical qerror
      character*4 atonam(*), resnam(*)
      character*1 ssusel
cLocal:
      integer nerr, iatom, irs, irsprev,
     .        i, j, k, ii
      double precision x, y, z, bfcol
      character*1  ssu
      character*4  ren
      character*5  atncur
      character*80 lign80
cBegin:
      write(6,'(/A)') ' Rdatompdb> Reading pdb file.'
c
      qerror=.false.
      nerr=0
c
      irsprev=-1
      nres=0
      iatom=0
 105  continue   
      read(unpdb,'(A)',end=200,err=110) lign80 
c 
      goto 120                                
 110  continue
      nerr=nerr+1                            
c
 120  continue                              
      if (lign80(1:4).eq.'ATOM') then
      read(lign80,'(12X,A4,1X,A4,A1,I4,4X,3F8.3,6X,F6.2)') 
     .            atncur, ren, ssu, irs, x, y, z, 
     .            bfcol
      if (iatom.lt.natmax) then
          if (ssu.eq.ssusel.or.ssusel.eq.' ') then
          iatom=iatom+1
          xat(iatom)=x
          yat(iatom)=y
          zat(iatom)=z
          binfo(iatom)=bfcol
c
          call stringcl(atncur,lnom)
          atonam(iatom)=atncur
          call stringcl(ren,lnom)
          resnam(iatom)=ren
          iresat(iatom)=irs
c
          if (irs.ne.irsprev) then
              nres=nres+1
              if (nres.gt.nresmx) then
                  write(6,'(A/A,I6)') 
     .          '%Rdatompdb-Err> Too many residues in this file.',
     .          ' Maximum allowed is = ',nresmx
                  stop
              endif
              irsprev=irs
              fatres(nres)=iatom
          endif
          endif
      else
          write(6,'(A/A,I6)') 
     .      '%Rdatompdb-Err> Too many atoms in this file.',
     .      ' Maximum allowed is = ',natmax
          stop
      endif
      endif
c
c     2) Ligne suivante du fichier pdb :
c
      goto 105
c
c     3) Fin de la lecture du fichier pdb :
c
 200  continue 
      write(6,*) 'Rdatompdb> End of file reached.'
      write(6,*) 'Rdatompdb> Number of I/O errors: ',
     .            nerr
c
      natom=iatom
      fatres(nres+1)=natom+1
      irs=0
      if (natom.gt.0) irs=iresat(natom)
c
      write(6,'(/(A,I6))') 
     .' Rdatompdb> Number of residues found = ',nres,
     .'            First residue number     = ',iresat(1),
     .'            Last  residue number     = ',irs,
     .'            Number of atoms found    = ',natom
      write(6,'(A,F8.1)') 
     .'            Mean number per residue  = ',float(natom)/float(nres)
c
      if (natom.eq.0) then
          write(6,'(A)')
     .  '%Rdatompdb-Err> No atom found in file.'
          qerror=.true.
      endif
      if (nres.eq.0) then
          write(6,'(A)')
     .  '%Rdatompdb-Err> No residue found in file.'
          qerror=.true.
      endif
c
      return
      end

c     Questions/reponses.
c     Serie de routines utilitaires:
c     getnam
c     getnum
c     getchi
c     Version 1.4, Bordeaux.
c
      subroutine getchi(message,numlu,qok)
c
c     NUMLU obtenu en reponse au MESSAGE.
c     NTRYMX essais en cas de probleme.
c     YHS-oct-96
c
      implicit none
cI/O:
      double precision numlu
      logical qok
      character*(*) message
cLocal:
      integer ntry, ntrymx
      double precision iread
cBegin:
      ntrymx=5
c
      qok=.false.
      ntry=0
c
 100  continue
      ntry=ntry+1
      if (ntry.ge.ntrymx) return
c
      write(6,'(A,A)') ' Getchi> ',message
      read(5,*,end=200,err=100) iread
      numlu=iread
c
      write(6,*) 'Getchi> ',numlu
c
      qok=.true.
      return
 200  continue
      return
      end 
c---------------------------------------------------
      subroutine getnam(message,nomlu,lnomlu,qok)
c
c     NOMLU obtenu en reponse au MESSAGE.
c     NTRYMX essais en cas de probleme.
c     YHS-oct-96
c
      implicit none
cI/O:
      integer lnomlu
      logical qok
      character*(*) message, nomlu
cLocal:
      integer ntry, ntrymx
cBegin:
      ntrymx=5
c
      qok=.false.
      ntry=0
c
 100  continue
      ntry=ntry+1
      if (ntry.ge.ntrymx) return
c
      write(6,'(A,A)') ' Getnam> ',message
      read(5,'(A)',end=200,err=100) nomlu
c
      call stringcl(nomlu,lnomlu)
      write(6,'(A,A)') ' Getnam> ',nomlu(1:lnomlu)
c
      qok=.true.
      return
 200  continue
      return
      end 
c---------------------------------------------------
      subroutine getnum(message,numlu,nummin,nummax,qcheck,qok)
c
c     NUMLU obtenu en reponse au MESSAGE.
c
c     qcheck=.true. =>
c     NUMLU doit etre inferieur a nummax et superieur a nummin.
c
c     NTRYMX essais en cas de probleme.
c     qok=.false. => Probleme a la lecture.
c
c     YHS-oct-1996: version 1.0
c     YHS-jan-2001: version 4.0
c
      implicit none
cI/O:
      integer numlu, nummax, nummin
      logical qok, qcheck
      character*(*) message
cLocal:
      integer ntry, ntrymx, iread
cBegin:
      ntrymx=5
c
      qok=.false.
      ntry=0
c
 100  continue
      ntry=ntry+1
      if (ntry.ge.ntrymx) return
c
      write(6,'(A,A)') ' Getnum> ',message
      read(5,*,end=200,err=100) iread
      numlu=iread
c
      write(6,*) 'Getnum> ',numlu
c
      if (qcheck) then
      if (numlu.gt.nummax) then
          write(6,'(A,I6,A)') 
     .  '%Getnum-Err: number larger than ',nummax,
     .  ' This is not allowed. Sorry.'
          numlu=nummax
          return
      else if (numlu.lt.nummin) then
          write(6,'(A,I6,A)') 
     .  '%Getnum-Err: number smaller than ',nummin,
     .  ' This is not allowed. Sorry.'
          numlu=nummin
          return
      endif
      endif
c
      qok=.true.
      return
 200  continue
      return
      end 
